;
;********************************************************************
;                  Binary To BCD Conversion Routine
;      This routine converts a 16 Bit binary Number to a 5 Digit
;      This routine converts a 24 Bit binary Number to an 8 Digit
; BCD Number. This routine is useful since PIC16C55 & PIC16C57
; have  two 8 bit ports and one 4 bit port ( total of 5 BCD digits)
;
;       The 16 bit binary number is input in locations H_byte and
; L_byte with the high byte in H_byte.
;       The 5 digit BCD number is returned in R0, R1 and R2 with R0
; containing the MSD in its right most nibble.
;
;   Performance :			16 bit	24 bit
;               Program Memory  :       35	
;               Clock Cycles    :       885	2100 approx
;
; This file is based on the file BCD.ASM found on www.crownhill.co.uk
;
;*******************************************************************;
;
;

B2_BCD  bcf     STATUS,0                ; clear the carry bit
	movlw   .32	; number of bits to convert.
	movwf   BCDcount
	clrf    BCDop0
	clrf    BCDop1
	clrf    BCDop2
	clrf    BCDop3
	clrf    BCDop4

BCDloop16
	bcf	STATUS,C
	btfsc	BCDip3,7
	bsf	STATUS,C
	rlf     BCDip0
	rlf     BCDip1
	rlf     BCDip2
	rlf     BCDip3
	
	rlf     BCDop0
	rlf     BCDop1
	rlf     BCDop2
	rlf     BCDop3
	rlf     BCDop4
;
	decfsz  BCDcount
	goto    BCDadjDEC
	RETLW   0
;
BCDadjDEC  
;	movlw   BCDop4
;	movwf   FSR
;	call    BCDadjBCD

	movlw   BCDop0
	movwf   FSR
	call    BCDadjBCD

	movlw   BCDop1
	movwf   FSR
	call    BCDadjBCD
;
	movlw   BCDop2
	movwf   FSR
	call    BCDadjBCD
;
	movlw   BCDop3
	movwf   FSR
	call    BCDadjBCD
;
	goto    BCDloop16
;
BCDadjBCD  
	movlw   0x03
	addwf   0,W
	movwf   BCDtemp
	btfsc   BCDtemp,3       ; test if result > 7
	movwf   0
	movlw   0x30
	addwf   0,W
	movwf   BCDtemp
	btfsc   BCDtemp,7       ; test if result > 7
	movwf   0               ; save as MSD
	RETLW   0

