;**********************************************************
;*  fc002.asm
;**********************************************************
;*  This program configures a PIC 12F675 or 12F683 to measure frequency.
;*  It uses a count of cycles over a period.
;* 
;* 	Set up the comparator and Vref for the freq input
;*	
;*	Analogue input = GP1
;*    test input, GP3, 
;*		if GP3 is high, then GP2 is Freq counter input.
;*		if GP3 is low,  then GP1 is Freq counter input. It uses the Comparator and Vref set to about 1 volt.
;*
;*	The frequency is measured other a two second period.
;*	The frequency to be measured clocks the TMR0 divided by two over exactly 2 seconds timed by cycle counting.
;*	
;*	For each sample the amount that TMR0 has incremented in at worst 30 cycles.
;*
;*	This means that the max frequency is 256 cycles in 30 uSec = 256 Mhz / 30  = 8.5 Mhz max
;*
;*	I have not tried this counter above 115 Khz, with and oscillator.
;*	
;* example measurements takend from a Maplin frequency generator. This was one of teh sine wave outputs that was very jittery.
;* another frequency meter gave similar readings. I was using intOsc, which was not calibrated. 
;* f(0,0E,00129100,0);
;* f(0,0E,00129093,0);
;* f(0,0E,00129060,0);
;* f(0,0E,00129005,0);
;* f(0,0A,00129112,0);
;* f(0,0A,00129247,0);
;* f(0,0A,00129066,0);
;* f(0,0E,
;*
;* it is measuring 129kHz using the comparator.
;*
; Measurements of the scope oscillator via the 12F675's comparator. At 2Mhz, the measurement aliased
;*
;f(0,0E,01996579,0);
;f(0,0A,01996546,0);
;f(0,0E,01996652,0);
;f(0,0E,01996740,0);
;f(0,0A,01996722,0);
;f(0,0A,01996768,0);
;f(0,0A,01996771,0);
;f(0,0A,

; The TMR0 is syncronized to the system clock, so that reads of TMR0 avoid count ripples. This limits the counter to a max frequency, This code sets the prescaler to 1:2
;
;/*
;5.1.5 USING TIMER0 WITH AN
;EXTERNAL CLOCK
;When Timer0 is in Counter mode, the synchronization
;of the T0CKI input and the Timer0 register is accomplished
;by sampling the prescaler output on the Q2 and
;Q4 cycles of the internal phase clocks. Therefore, the
;high and low periods of the external clock source must
;meet the timing requirements as shown in the
;Section 15.0 Electrical Specifications.
;*/
;*
;* 	The prescaler is set to divide by 2. I have seen PIC frequency counters rated to 50 MHz using the prescaler.
;*
;*	Connect GP0 direct to the serial input of your PC set to 1200 baud.
;*
;*	It outputs f(0,0A,00116074,0); 
;* 		where the 2nd Paramter in the state of the GPIO, 
;*		and the 3rd parameter is the frequency in decimal hertz. 
;*
;*  Copyright 2005,2016 Douglas Rice
;*
;*
;**********************************************************
;There are one or two minor issues found using MPLAB v8.63
; 1) The serial output does not appear on GP0 
;    until after the first measurement. 
;    Added code to disable analogue 
;
; 2) I only output one stop bit, it might be better to send two, 
;    otherwise PUTTY needs the Serial set to 7 bits.
;    Added some more stop bits
;
;**********************************************************
;
;
;
;**********************************************************

;--------------------------------------------------------------------------
; Sec 1.0	Build support for 12F675 or 12F683
;-------------------------------------------------------------------------
; uncomment one of these below or define it on the command line
;#define using12F675
;#define using12F683


#ifdef using12F675
	list p=12f675
	; Include file, change directory if needed
	include "p12f675.inc"
	__idlocs  0xF675	
; Using internal 4Mhz Osc	
;	__config  _PWRTE_ON & _WDT_OFF & _BODEN_OFF & _MCLRE_OFF & _INTRC_OSC_NOCLKOUT
	__config  _PWRTE_ON & _WDT_OFF & _BODEN_OFF & _MCLRE_OFF & _INTRC_OSC_CLKOUT

; Using external 4Mhz Osc	
;	__config  _PWRTE_ON & _WDT_OFF & _BODEN_OFF & _MCLRE_OFF & _XT_OSC
#endif


#ifdef using12F683
	list p=12f683
	; Include file, change directory if needed
	include "p12f683.inc"
	__idlocs  0xF683
; NOTE:- 12F683 has failsafe clock so can run off internal clock or external clock.
; MORE WORK IS REQUIRED TO SET THIS UP.
; Need to set up OSCON to select external oscillator. This is to be done.
; Using internal 4Mhz Osc if FCMEM_ is not diabled explicitly.	
;	
; Using internal 4Mhz Osc	
;	__config  _PWRTE_ON & _WDT_OFF & _BOD_OFF & _MCLRE_OFF & _INTRC_OSC_NOCLKOUT
	__config  _PWRTE_ON & _WDT_OFF & _BOD_OFF & _MCLRE_OFF & _INTRC_OSC_CLKOUT

; Using external 4Mhz Osc	
;	__config  _PWRTE_ON & _WDT_OFF & _BOD_OFF & _MCLRE_OFF & _XT_OSC

;
; 12F683 has extra CMCON1 for TMR1 sync
;
#define	CMCON CMCON0
 
#endif

;--------------------------------------------------------------------------
; Sec 1.2	Variables
;----1---------------------------------------------------------------------

	cblock 	0x20
	  ; variables used for context saving 
	  w_temp
	  status_temp
	  ADrh
	  ADrl
	  ADcnt

	  SRtxTemp
	  SRtxCnt
	  SRout
	  SRdel


	  cnt1
	  cnt2
	  cnt3
	  		
	endc
	
	cblock		; variables used in BCD.INC
	BCDip0	
	BCDip1	
	BCDip2	
	BCDip3
	
	BCDop0                    ; RAM Assignments
	BCDop1      
	BCDop2      
	BCDop3      
	BCDop4      

	BCDcount  
	BCDtemp   
	endc

;--------------------------------------------------------------------------
; Sec 1.4	MACRO for tables straddling memory page
;-------------------------------------------------------------------------

TEST_STRADDLE	MACRO	START
	if high( $ ) != high( START ) 
	    Error "Table straddles Page Boundary "
	endif
	endm


	; Start at the reset vector
	org	0x000
	goto	Init



;--------------------------------------------------------------------------
; Sec 2.0	Code
;-------------------------------------------------------------------------

Init

	org	0x010

#ifdef using12F675	
;	banksel 0x3FF
;	CALL 	0x3FF
	MOVWF	OSCCAL
#endif
#ifdef using12F683
	nop
;TBD
#endif


	banksel	TRISIO 

;	BSF		STATUS, RP0
	movlw	0xDF
	MOVWF	TRISIO		; Set output high to start Measurement
	BCF		STATUS, RP0

;;;;;;;;;;;;;;;;;;;
	; Configure the Comparator to low power
	; for 12F683 #define	CMCON CMCON0
	banksel  CMCON
	movlw	B'00010111'	;
	movwf	CMCON

	; turn off A to D converter
	banksel  ADCON0
	movlw	B'00000000'	;
	movwf	ADCON0

	banksel  ANSEL
	movlw	B'00000000'	;Fosc/8, A/D disabled
	movwf	ANSEL

	; Set up the comparator and Vref for the freq input	
;	Analogue input = GP1
;	r19;=43 cmcom
;	r99;=88	vrcon
;	r9f;=00	adsel
;	r05;=15 gpio
;	r85;=3b	trisio

	banksel  VRCON
	movlw	1 << VREN | 0<<VRR | 5	; level = Vdd * 4 / 24 = 1.0 Volts if Vdd is 5 volts.
	movwf	VRCON
;;;;;;;;;;;;;;;;;;;


Start
	; output f(0,freq,0);

;	movlw	0xFF
;	call	SRtxChar

;	movlw	0xFF
;	call	SRtxChar

;	movlw	0x55
;	call	SRtxChar

;	movlw	0x55
;	call	SRtxChar

;	movlw	0xAA
;	call	SRtxChar

;	movlw	0xAA
;	call	SRtxChar

;	movlw	0xFF
;	call	SRtxChar

;	movlw	0xFF
;	call	SRtxChar

;	movlw	0x00
;	call	SRtxChar

;	movlw	0x00
;	call	SRtxChar

	movlw	' '
	call	SRtxChar


	movlw	0x0D
	call	SRtxChar

	movlw	0x0A
	call	SRtxChar

	movlw	'f'
	call	SRtxChar
	movlw	'('
	call	SRtxChar

	movlw	'0'
	call	SRtxChar

	movlw	','
	call	SRtxChar
	call	FRfreqMeter

	movlw	','
	call	SRtxChar
	movlw	'0'
	call	SRtxChar

	movlw	')'
	call	SRtxChar

	movlw	';'
	call	SRtxChar

	goto	Start		;Do it again


;--------------------------------------------------------------------------
; Sec 2.1	Serial code - originally for PIC16F84 clocked at 32kHz
;-------------------------------------------------------------------------


SRwrtHexNibble	; currently only displays 0..9, A..F
	ANDLW	0x0F
	ADDLW	0x06	; is it A..F, if so trigger a digit overflow
	SKPNDC
	ADDLW	7; subtract 10, then add 'A'-'0'
	ADDLW	0x30-6	; Subtract extra 6 added to cause DC
	MOVWF	SRout
	GOTO	SRtxChar



SRtxChar
	;
	;
	;Output Start Bit
	;
	;Output LSB first
    ;
    ; -/=/=/=/=/=/=/=/=/=/__
	; START/lsb...msb/stop
	;
	; The PIC pin is directly connected to PIN2 of the RS232 socket
    ; Normally a MAX232 would invert the signal
    ; so Start is 1, the bits are sent LSB first and the stop is 0
    ;
    ;http://www.lookrs232.com/rs232/waveforms.htm
    ;
    ;


TxOnBit  equ 0
	
	movwf	SRtxTemp
	comf	SRtxTemp,f

	
	BCF		GPIO,TxOnBit
	MOVLW	~( 1 << TxOnBit ) 
	
;	TRIS	GPIO
;	BSF		STATUS, RP0
	banksel TRISIO
	MOVWF	TRISIO		; Set output high to start Measurement
;	BCF		STATUS, RP0

	banksel GPIO
	call	delay	

	; 1200 baud output
	; move the char into SRtxtemp, it is destroyed.
	;Start Bit
	BSF		GPIO,TxOnBit
	GOTO	$+1
	call	delay	
	movlw	4
	movwf	SRtxCnt
	GOTO 	$+1
SRtxCharLp1

	; This outputs two bits at a time 
	; this takes 6.5 cycles each when using osc =32768Hz and 1200 baud.
	
	; After start bit, which is 1, then test each bit
	; I need to set PA:0 to same as LSB SRtxTemp
	; this compares PA:0 with SRtxTemp so see if it needs toggling
	; The xorwf PORTA,f causes the bit to be toggled if required.

	; Toggle bit if next bit is different 
	RRF		SRtxTemp,f
	movfw	STATUS
	andlw	1 << C		; mask off bit
	addlw   ( 1 << TxOnBit) -1	; shift bit by using a ripple carry
		
	xorwf	GPIO,W
	andlw	0x1 << TxOnBit
	xorwf	GPIO,f		;t+6,19

	GOTO	$+1	
	call	delay	
	RRF		SRtxTemp,f

	movfw	STATUS
	andlw	1 << C
	addlw   ( 1 << TxOnBit) -1

	xorwf	GPIO,W
	andlw	0x1 << TxOnBit
	xorwf	GPIO,f		;t+11,24
	GOTO	$+1	
	call	delay	
	decfsz	SRtxCnt
	goto	SRtxCharLp1
	GOTO	$+1	
	GOTO	$+1	
	; output stop bit
	BCF		GPIO,TxOnBit
	call	delay	
	GOTO	$+1	
	GOTO	$+1	

	; add some more stop bits
	BCF		GPIO,TxOnBit
	call	delay	
	GOTO	$+1	
	GOTO	$+1	

	BCF		GPIO,TxOnBit
	call	delay	
	GOTO	$+1	
	GOTO	$+1	
	RETURN


delay	
	; one bit delay
	nop
;	movlw	0x03  ; delay for 32K

	movlw	0xCD  ; delay for 1200K
	movwf	SRdel
delay1	
	NOP
	decfsz	SRdel
	goto	delay1
	return


;--------------------------------------------------------------------------
; Sec 4.1	Frequency Counter Code
;-------------------------------------------------------------------------
	cblock
		current
		lasttmr0
		;BCDip0,BCDip1,BCDip2,BCDip3
		tc0,tc1,tc2,tc3,tc4
	endc

	
incCnt	
	; get current count
	; how much has TMR0 incremented,
    ; Add this to the BCDip
    ;
	movfw	current
	movwf	lasttmr0
	movfw	TMR0

; the loop counter needs to be checked that this read is two seconds after the first read of TMR0
	movwf	current
	subwf	lasttmr0,w
	; we don't need to worry about the carry.
	sublw	0
	addwf	BCDip0,f		; t
	skpc			; t+0 
	retlw	-.24		; t+1   these values are used to adjust the main timing loop.
	incfsz	BCDip1,f		; t+2
	retlw	-.26		; t+3
	incfsz	BCDip2,f		; t+4
	retlw	-.28		; t+5
	incfsz	BCDip3,f		; t+6
	retlw	-.30		; t+7
	

FRfreqMeter1
	; we arm to display first of all
	NOP
	return

FRfreqMeter

	
	; turn off A to D converter
	banksel  ADCON0
	movlw	B'00000000'	;
	movwf	ADCON0

	banksel  ANSEL
	movlw	B'00000000'	;Fosc/8, A/D disabled
	movwf	ANSEL


	; Set up the comparator and Vref for the freq input
	
;	Analogue input = GP1


;	r19;=43 cmcom
;	r99;=88	vrcon
;	r9f;=00	adsel
;	r05;=15 gpio
;	r85;=3b	trisio

	banksel  VRCON
	movlw	1 << VREN | 0<<VRR | 5	; level = Vdd * 4 / 24 = 1.0 Volts if Vdd is 5 volts.
	movwf	VRCON


	banksel  GPIO
	call	delay
	movlw	'0'
	call	SRtxChar
	movlw	'x'
	call	SRtxChar

	banksel	GPIO
	swapf	GPIO,W		;
	call 	SRwrtHexNibble
	movf	GPIO,W		;
	call 	SRwrtHexNibble
	movlw	','
	call	SRtxChar



	; test input, GP3, if High then GP2 is input and freq counter.
	BTFSC	GPIO,3
	goto	FRfreqMeterNoComparator


FRfreqMeterComparator	
	banksel	TRISIO
	movlw	B'11111011'	;Fosc/8, A/D enabled
	movwf	TRISIO

	; Make sure that the pin is not used by the A to D 
	; Make sure that the Comparator input is Analogue, but the output is digital

	; Configure the Comparator to Use Vref and GPIO pin 1
	; for 12F683 #define	CMCON CMCON0
	banksel  CMCON
	movlw	B'00010011'	;
	movwf	CMCON

	goto	FRfreqMeter2	


FRfreqMeterNoComparator
	; Make sure that the pin is not used by the A to D 
	; Make sure that the Comparator inpuit is Analogue, but the output is digital

	; make the GPIO all inputs
	banksel  TRISIO
	movlw	B'11111111'	;Fosc/8, A/D enabled
	movwf	TRISIO

	; Configure the Comparator to low power
	; for 12F683 #define	CMCON CMCON0
	banksel  CMCON
	movlw	B'00010111'	;
	movwf	CMCON

	; make the GPIO all inputs
	banksel  TRISIO
	movlw	B'11111111'	;Fosc/8, A/D enabled
	movwf	TRISIO

FRfreqMeter2

	
	; Change Prescaler and connect TMR0 ip to external
	CLRWDT
	; Configure Tmr 0
	BSF	STATUS,RP0
	; set TMR0 to increment off internal input for test purposes - CLK off internal Clock
	MOVLW	0x0 + 0x00  + 0 	; T0CS + For Tmr0 0=/2, 1=/4, 2=/8,3=/16

	; Set up CLK off external input with prescale set to 1/2.
	MOVLW	0x0 + 0x20  + 0 	; T0CS + For Tmr0 0=/2, 1=/4, 2=/8,3=/16


	MOVWF	OPTION_REG
	BCF	STATUS,RP0

	clrf	BCDip0
	clrf	BCDip1
	clrf	BCDip2
	clrf	BCDip3
	; call delay to allow comparator to settle down
	call	delay
	call	delay
	call	delay
	call	delay
	call	delay
	call	delay
	call	delay
	call	delay
	call	delay
	call	delay


;-- WARNING ------------------------------------------------------
    ; start measuring frquency HERE as we sample the count
    ; adding instructions here affects the readings
	movfw	TMR0
	movwf	current

    nop

	; 2,000,000 = 1E8480
	; using prescale of div 2
	; These seem to work, 
	; but still underead a litte bit
	movlw	0x81
	movwf	tc0
	movlw	0x85
	movwf	tc1
	movlw	0x1F
	movwf	tc2

freqMeter0
	nop
	;	21 cycles
	call	incCnt

	nop

FreqDelay
	; w contains a fiddle factor to adjust this delay
	; get current count 
	
	; we don't need to worry about the carry.
	addwf	tc0,f		;t
				;t= 0
	skpnc
	goto	fd1		;t+1 these values are used to adjust the main timing loop.
				;
	decfsz	tc1,f		;t+2
	goto	fd2		;t+3
	decfsz	tc2,f		;t+4
	goto	fd3		;t+5
	goto 	fd4


	
fd1	
	; replace NOP instructions with a test to see if buttons have been pressed
	; note that theses also take 2 clock cycles

	nop			;t+5
	nop			;t+6
f
fd2
	nop			;t+5
	nop			;t+6

fd3	goto	freqMeter0	;t+7
fd4	goto	FRdisplayResult
	; we have a reading so arm to display it.

fdEscape ; come here if escaped out offrequency measurement.
	; The half measured frequency should not be displayed.


	; Change Prescaler back to TMR0 clocked by Internal.
	CLRWDT
	; Configure Tmr 0
	BSF	STATUS,RP0
	; Set up prescaller for 8192 / 16 = 512 ticks perseconds.
	MOVLW	0x0 + 2 	; For Tmr0 0=/2, 1=/4, 2=/8,3=/16
	MOVWF	OPTION_REG
	BCF	STATUS,RP0
	; now run into the convert and display routine
	; display current count


FRdisplayResult1
	; we arm to display first of all
	clrf	BCDip0
	clrf	BCDip1
	clrf	BCDip2
	clrf	BCDip3

FRdisplayResult

	; make the GPIO all inputs
	banksel  TRISIO
	movlw	B'11111111'	;Fosc/8, A/D enabled
	movwf	TRISIO

	; Configure the Comparator to low power
	; for 12F683 #define	CMCON CMCON0
	banksel  CMCON
	movlw	B'00000111'	;
	movwf	CMCON
	
	; test input, GP3, if High then GP2 is input and freq counter.
	BTFSC	GPIO,3
	goto	FRfreqMeterNoComparator2


FRfreqMeterComparator2	
	banksel	TRISIO
	movlw	B'11111011'	;Fosc/8, A/D enabled
	movwf	TRISIO

	; Make sure that the pin is not used by the A to D 
	; Make sure that the Comparator input is Analogue, but the output is digital

	; Configure the Comparator to Use Vref and GPIO pin 1
	; for 12F683 #define	CMCON CMCON0
	banksel  CMCON
	movlw	B'00010011'	;
	movwf	CMCON
	goto	FRfreqMeter3	


FRfreqMeterNoComparator2
	; Make sure that the pin is not used by the A to D 
	; Make sure that the Comparator inpuit is Analogue, but the output is digital

	; make the GPIO all inputs
	banksel  TRISIO
	movlw	B'11111111'	;Fosc/8, A/D enabled
	movwf	TRISIO

	; Configure the Comparator to low power
	; for 12F683 #define	CMCON CMCON0
	banksel  CMCON
	movlw	B'00010111'	;
	movwf	CMCON

	; make the GPIO all inputs
	banksel  TRISIO
	movlw	B'11111111'	;Fosc/8, A/D enabled
	movwf	TRISIO

FRfreqMeter3
	
	


;--------------------------------------------------------------------------
; Sec 5.1	Binary to BCD
;-----------------------------------------------------------------

#DEFINE	buildbcdBinDec24
;	BCF	PORTA,0

;	call	bcdBinDec24
	banksel	BCDop1
	call    B2_BCD          ; After conversion the Decimal Number

CNdispCount
	; now set up the display for the display.

	SWAPF	BCDop3,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop3
	CALL	SRwrtHexNibble

	SWAPF	BCDop2,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop2
	CALL	SRwrtHexNibble

	SWAPF	BCDop1,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop1
	CALL	SRwrtHexNibble

	SWAPF	BCDop0,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop0
	CALL	SRwrtHexNibble


	; we have displayed it so prevent another display
	return


;--------------------------------------------------------------------------
; Sec 5.2	Binary to BCD -Procedure
;-------------------------------------------------------------------------
	include	"bcd.inc"



	end


