;**********************************************************
;*  fc004.asm - Event Logger 
;**********************************************************
;*
;*  Copyright 2005,2016 Douglas Rice
;*
;*  http://www.dougrice.plus.com/hp/freqCounter/freqCounter.html
;*
;**********************************************************
;*  This uses the file BCD.ASM found on www.crownhill.co.uk and in 
;*  http://ww1.microchip.com/downloads/en/appnotes/00544d.pdf
;*; and 
;*  http://ww1.microchip.com/downloads/en/AppNotes/00526e.pdf 
;*  
;*  Copyright 1997 Microchip 
;**********************************************************
;*  Event logger:-
;*
;*				+-----------------------+
;*   10MHz/4->--+ 2   TMR1 +16 bit      + --> TMR1IF  
;*              +-----------------------+
;*                       |
;*              +-----------------------+
;*   event  ->--+ 5  CCP  32bit Capture + 
;*              +-----------------------+
;*                       |
;*                       | on CCPIF event
;*                       |
;*              +-----------------------+
;*              +       bin >BCD        + 
;*              +-----------------------+
;*                       |
;*              +-----------------------+
;*              +     Serial            + --> 19200 baud RS232 ->Pin2
;*              +-----------------------+
;*
;**********************************************************
;*  This code uses the Microchip 12F683 to timestamp an event.
;* 
;*  The CCP module is configured to take a time stamp on the rising edge of the event signal connected to Pin 5.
;*  
;*  The time stamp, in units of 0.4us, is output at 19200 baud on pin 7. This connects to the RS232 rx pin, pin 2 of 9pin connector. 
;*
;*	The time stamps are 32bits so range from 0 to 2^32 or  0 to 4294967296. 
;*
;*	The time stamp rolls over after 2^32/2500000 seconds = 1717.9869184 seconds = 28.633115306666666666666666666667 minutes
;*
;*  The chip is clocked at 10Mhz. 
;*  By using the XT oscillator mode, the OSC 1 pin can be driven by a Frequency standard with a sinewave 10dBm output. 
;*
;*  TMR1 clocks at Fosc/4 or 2.5Mhz. This is a 16 bit counter. 
;*  When TMR1 overflows it increments another 16 Bit counter. 
;*  This provides a time stamp between 0 to 2^32 or 0 to 4294967296.
;*
;**********************************************************
;* Example output:-
;* 
;* event 0.4us
;* 
;* 0031229812
;* 0031279812
;* 0031329812
;* 0031379812
;* 0031429812
;* 0031479812
;**********************************************************
;* Example of captured output converted to difference, Milleseconds and Frequence using a spreadsheet.
;*
;*  Use the formulae below to get the difference
;*  =+B9-B8
;*  
;*  Use the formulae below to convert the difference to milli seconds
;*  =+C9/2500
;*
;* CCP	      delta CCP	ms	    Hz
;*======================================
;* 3339022700			
;* 3339072700	50000	20	    50
;* 3339122701	50001	20.0004	49.99900002
;* 3339172701	50000	20	    50
;* 3339222701	50000	20	    50
;* 3339272701	50000	20	    50
;* 3339322701	50000	20	    50
;*
;**********************************************************
;*  See a similar device, the PICPET (  http://www.leapsecond.com/pic/picpet.htm )
;*
;*	The PIC PET is an event logger, It outputs a time stamp between 0 and 100 seconds. in 0.4us steps.
;*
;*  Limitations:-
;*  The time to output the time stamp is about 10 ms, so the events need to be greater than 10ms apart. Keep the event rate less than 100Hz.
;*
;*  Copyright 2005,2016 Douglas Rice
;*
;*  http://www.dougrice.plus.com/hp/freqCounter/freqCounter.html
;*
;**********************************************************
;*  Event logger:-
;*
;*				+-----------------------+
;*   10MHz/4->--+ 2   TMR1 +16 bit      + --> TMR1IF  
;*              +-----------------------+
;*                       |
;*              +-----------------------+
;*   event  ->--+ 5  CCP  32bit Capture + 
;*              +-----------------------+
;*                       |
;*                       | on CCPIF event
;*                       |
;*              +-----------------------+
;*              +       bin >BCD        + 
;*              +-----------------------+
;*                       |
;*              +-----------------------+
;*              +     Serial            + --> 19200 baud RS232 ->Pin2
;*              +-----------------------+
;*
;**********************************************************
;*
;--------------------------------------------------------------------------
; Sec 1.0	Build support for 12F683
;-------------------------------------------------------------------------
; uncomment this below or define it on the command line
;
#define using12F683

#ifdef using12F683
	list p=12f683
	; Include file, change directory if needed
	include "p12f683.inc"
	__idlocs  0xF683	

; NOTE:- 12F683 has failsafe clock so can run off internal clock or external clock.
; MORE WORK IS REQUIRED TO SET THIS UP.
; Need to set up OSCON to select external oscillator. 
; Using internal 4Mhz Osc if FCMEM_ is not diabled explicitly.	
;
;_FCMEN_ON                    EQU     H'3FFF'
;_FCMEN_OFF                   EQU     H'37FF'

;	__config  _PWRTE_ON & _WDT_OFF & _BOD_OFF & _MCLRE_OFF & _INTRC_OSC_NOCLKOUT
;	__config  _PWRTE_ON & _WDT_OFF & _BOD_OFF & _MCLRE_OFF & _INTRC_OSC_CLKOUT

; Using 10Mhz External clock with 
;	__config  _PWRTE_ON & _WDT_OFF & _BOD_OFF & _MCLRE_OFF & _EC_OSC & _FCMEN_OFF & _IESO_OFF

; Using 10Mhz  XT OSC with External clock, works   Set 10Mhz to 9.4 Mhz then 10.3Mhz and serial was readable. 
	__config  _PWRTE_ON & _WDT_OFF & _BOD_OFF & _MCLRE_OFF & _XT_OSC & _FCMEN_OFF & _IESO_OFF

; Using 10Mhz  HS OSC with External clock, did not work with sine wave output.  
;	__config  _PWRTE_ON & _WDT_OFF & _BOD_OFF & _MCLRE_OFF & _HS_OSC & _FCMEN_OFF & _IESO_OFF

; Using 10Mhz  LP OSC with External clock, works 4Vpp on scope  - NOT DESIGNED FOR 10MHz
;	__config  _PWRTE_ON & _WDT_OFF & _BOD_OFF & _MCLRE_OFF & _LP_OSC & _FCMEN_OFF & _IESO_OFF

;
; 12F683 has extra CMCON1 for TMR1 sync
;
#define	CMCON CMCON0

#endif
;--------------------------------------------------------------------------
; Sec 1.2	Variables
;----1---------------------------------------------------------------------

	cblock 	0x20
	  ; variables used for context saving 
	  w_temp
	  status_temp

	  SRtxTemp
	  SRtxCnt
	  SRout
	  SRdel

	  ; copy CCP cature
	  tmpCcp1L
	  tmpCcp1H

	  ; 32 bit capture 
	  lastCcp1L
	  lastCcp1H
	  lastCcp1TmrOV0
	  lastCcp1TmrOV1

	  ; Add a 16bit counter to count TMR1 overflows. 	
	  tmr1overflow0
	  tmr1overflow1
	  
	  ; save PIR if a CCPIF or TMRIF is set.
	  lastPIR 
	
;	endc
;	cblock		; variables used in BCD.INC
	; Store Binary into here
	BCDip0	
	BCDip1	
	BCDip2	
	BCDip3
	
	; BCD output to here.
	BCDop0                    ; RAM Assignments
	BCDop1      
	BCDop2      
	BCDop3      
	BCDop4      
	BCDop5      
	BCDop6      

	BCDcount  
	BCDtemp   
	endc

;--------------------------------------------------------------------------
; Sec 1.4	MACRO for tables straddling memory page
;-------------------------------------------------------------------------

TEST_STRADDLE	MACRO	START
	if high( $ ) != high( START ) 
	    Error "Table straddles Page Boundary "
	endif
	endm

	; Start at the reset vector
	org	0x000
	goto	Init



;--------------------------------------------------------------------------
; Sec 2.0	Code
;-------------------------------------------------------------------------

Init

	org	0x010

#ifdef using12F683
	nop
;TBD
#endif

	banksel	TRISIO 

;	BSF		STATUS, RP0
	movlw	0xDF
	MOVWF	TRISIO		; Set output high to start Measurement
	BCF		STATUS, RP0

	; Configure the Comparator to low power
	; for 12F683 #define	CMCON CMCON0
	banksel  CMCON
	movlw	B'00010111'	;
	movwf	CMCON

	; turn off A to D converter
	banksel  ADCON0
	movlw	B'00000000'	;
	movwf	ADCON0

	banksel  ANSEL
	movlw	B'00000000'	;Fosc/8, A/D disabled
	movwf	ANSEL

	; Set up the comparator Vref
	banksel  VRCON
	movlw	0 << VREN | 0<<VRR | 5	; level = Vdd * 4 / 24 = 1.0 Volts if Vdd is 5 volts.
	movwf	VRCON
;;;;;;;;;;;;;;;;;;;

InitCCP
;
; set up TMR1 and CCP
;

;

	banksel	T1CON

	movlw	0 << T1GINV | 0 << T1GE  | 0 << T1CKPS0 | 0 << T1OSCEN | 0 << NOT_T1SYNC | 0 << TMR1CS | 0 << TMR1ON
    movwf	T1CON
	movlw	0 << T1GINV | 0 << T1GE  | 0 << T1CKPS0 | 0 << T1OSCEN | 0 << NOT_T1SYNC | 0 << TMR1CS | 1 << TMR1ON
    movwf	T1CON

;
; set up CCP to capture Signal and capture Period.
;

;
; If measuring 50Hz, period os 20ms which fits in the 16bit 
;

    banksel	CCP1CON
	movlw	0<<DC1B0 | B'0100' << CCP1M0 
    movwf	CCP1CON
	banksel	GPIO


;	movlw	' '
;	call	SRtxChar

; output CRLF
	movlw	0x0D
	call	SRtxChar

	movlw	0x0A
	call	SRtxChar

	movlw	'e'
	call	SRtxChar
	movlw	'v'
	call	SRtxChar
	movlw	'e'
	call	SRtxChar
	movlw	'n'
	call	SRtxChar
	movlw	't'
	call	SRtxChar
	movlw	' '
	call	SRtxChar

	movlw	'0'
	call	SRtxChar
	movlw	'.'
	call	SRtxChar
	movlw	'4'
	call	SRtxChar
	movlw	'u'
	call	SRtxChar
	movlw	's'
	call	SRtxChar


	movlw	0x0D
	call	SRtxChar

	movlw	0x0A
	call	SRtxChar

	clrf	tmr1overflow0
	clrf	tmr1overflow1

StartCCP

; output CRLF
	movlw	0x0D
	call	SRtxChar

	movlw	0x0A
	call	SRtxChar


;waitCCP
	; wait for the CCP
;	banksel PIR1
;	btfss   PIR1,CCP1IF
;	goto	waitCCP

waitCCPorTMR1

	; wait for the CCPiF or TMR1F
    ; there is a race condition here is capture is close to TMR1 overflow
    ; if capture is just before TMR1 roll over, but there is a delay in sampling PIR1
    ; if capture is close to roll over, then tmr1overflow may be out by one.
    ;  this needs more work 
    ; we also need a flog to time 
	banksel PIR1
    movfw	PIR1
	andlw	1<<CCP1IF | 1<<TMR1IF
	skpnz   
	goto	waitCCPorTMR1
	
	; reset active bit in PIR
	; xor W with PIR to toggle 0 xor 0 =0, 1 xor 1 = 0 
	xorwf	PIR1,f
	banksel	lastPIR
	movwf	lastPIR
    ;
    ; test if capture is just before roll over.
    ; ideally is capture just before 
    ;
    btfsc	lastPIR,TMR1IF
	call	inc_tmr1overflow	

    btfss	lastPIR,CCP1IF
	goto	waitCCPorTMR1	


	; Copy the time stamp. Copy the CCP and TMR1 overflow counter.

	banksel	CCPR1L
	movfw	CCPR1L
	banksel	lastCcp1L
	movwf	lastCcp1L
	movwf	BCDip0
 
	banksel	CCPR1H
	movfw	CCPR1H
	banksel	lastCcp1H
	movwf	lastCcp1H
	movwf	BCDip1

	movfw	tmr1overflow0
	movwf	lastCcp1TmrOV0
	movwf	BCDip2

	movfw	tmr1overflow1
	movwf	lastCcp1TmrOV1
	movwf	BCDip3

	; output in decimal
	; BCD takes 2ms
	call    B2_BCD          ; After conversion the Decimal Number

	; now set up the display for the display.
	; 65536 count ; @ 2.5M Hz or 400ns

	SWAPF	BCDop4,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop4
	CALL	SRwrtHexNibble



	SWAPF	BCDop3,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop3
	CALL	SRwrtHexNibble



	SWAPF	BCDop2,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop2
	CALL	SRwrtHexNibble



	SWAPF	BCDop1,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop1
	CALL	SRwrtHexNibble



	SWAPF	BCDop0,w
	CALL	SRwrtHexNibble ; 1.34 ms when 9600

	MOVFW	BCDop0
	CALL	SRwrtHexNibble

	goto	StartCCP

;--------------------------------------------------------------------------
; Sec 2.1	Functions 
;-------------------------------------------------------------------------

;
; When TMR1 overflows increment a 16bit counter
;
inc_tmr1overflow
    banksel	tmr1overflow0
    incfsz  tmr1overflow0,f
    return
    incfsz  tmr1overflow1,f
    return
	; we could
    return
	
;--------------------------------------------------------------------------
; Sec 2.1	Serial code - originally for PIC16F84 clocked at 32kHz
;-------------------------------------------------------------------------


SRwrtHexNibble	; currently only displays 0..9, A..F
	banksel	SRtxTemp
	ANDLW	0x0F
	ADDLW	0x06	; is it A..F, if so trigger a digit overflow
	SKPNDC
	ADDLW	7; subtract 10, then add 'A'-'0'
	ADDLW	0x30-6	; Subtract extra 6 added to cause DC
	MOVWF	SRout
	GOTO	SRtxChar

SRtxChar
	;
	;
	;Output Start Bit
	;
	;Output LSB first
    ;
    ; -/=/=/=/=/=/=/=/=/=/__
	; START/lsb...msb/stop
	;
	; The PIC pin is directly connected to PIN2 of the RS232 socket
    ; Normally a MAX232 would invert the signal
    ; so Start is 1, the bits are sent LSB first and the stop is 0
    ;
    ;http://www.lookrs232.com/rs232/waveforms.htm
    ;
    ;

TxOnBit  equ 0
	
	banksel	SRtxTemp
	movwf	SRtxTemp
	comf	SRtxTemp,f

	
	BCF		GPIO,TxOnBit
	MOVLW	~( 1 << TxOnBit ) 
	
;	TRIS	GPIO
;	BSF		STATUS, RP0
	banksel TRISIO
	MOVWF	TRISIO		; Set output high to start Measurement
;	BCF		STATUS, RP0

	banksel GPIO
	call	delay	

	; 9600 baud output
	; move the char into SRtxtemp, it is destroyed.
	;Start Bit
	BSF		GPIO,TxOnBit
	GOTO	$+1
	call	delay	
	movlw	8
	movwf	SRtxCnt
	GOTO 	$+1
SRtxCharLp1

	; This outputs one bits at a time 
	; this takes 6.5 cycles each when using osc =32768Hz and 1200 baud.
	
	; After start bit, which is 1, then test each bit
	; I need to set PA:0 to same as LSB SRtxTemp
	; this compares PA:0 with SRtxTemp so see if it needs toggling
	; The xorwf PORTA,f causes the bit to be toggled if required.

	; Toggle bit if next bit is different 
	RRF		SRtxTemp,f
	movfw	STATUS
	andlw	1 << C		; mask off bit
	addlw   ( 1 << TxOnBit) -1	; shift bit by using a ripple carry
		
	xorwf	GPIO,W
	andlw	0x1 << TxOnBit
	xorwf	GPIO,f		;t+6,19

	GOTO	$+1	
	call	delay	

	decfsz	SRtxCnt
	goto	SRtxCharLp1
	GOTO	$+1	
	GOTO	$+1	
	; output stop bit
	BCF		GPIO,TxOnBit
	call	delay	
	GOTO	$+1	
	GOTO	$+1	

	; add some more stop bits
	BCF		GPIO,TxOnBit
	call	delay	
	GOTO	$+1	
	GOTO	$+1	

	BCF		GPIO,TxOnBit
	call	delay	
	GOTO	$+1	
	GOTO	$+1	
	RETURN


delay	
	; one bit delay
;	nop

;   for 4Mhz Clock
;	movlw	0x03  ; delay for 32K
;	movlw	0x16  ; delay for 9600 baud = 104us ( 8us 
;	movlw	0xCD  ; delay for 1200 baud = 833 us	


;   for 10Mhz Clock, pad with nops as required
	nop	; 0.4us
;	nop 
;	nop
;	movlw	0x3c  ; delay for 9600 baud  = 104 us  
	movlw	.28   ; delay for 19200 baud = 52 us  
	movwf	SRdel
delay1	
	NOP
	decfsz	SRdel
	goto	delay1
	return


;--------------------------------------------------------------------------
; Sec 5.1	Binary to BCD
;-----------------------------------------------------------------

#DEFINE	buildbcdBinDec16
;	BCF	PORTA,0

;	call	bcdBinDec24
	banksel	BCDop1
	call    B2_BCD          ; After conversion the Decimal Number

CNdispCount
	; now set up the display for the display.
;   65,536

	SWAPF	BCDop3,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop3
	CALL	SRwrtHexNibble

	SWAPF	BCDop3,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop3
	CALL	SRwrtHexNibble

	SWAPF	BCDop2,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop2
	CALL	SRwrtHexNibble

	SWAPF	BCDop1,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop1
	CALL	SRwrtHexNibble


	SWAPF	BCDop0,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop0
	CALL	SRwrtHexNibble

	; we have displayed it so prevent another display
	return


;********************************************************************
; 2^32	= 42 94 96 72 96, overflows every 1717.9869184 seconds.
;********************************************************************

;--------------------------------------------------------------------------
; Sec 5.2	Binary to BCD -Procedure
;-------------------------------------------------------------------------
;	include	"bcd.inc"
;
;********************************************************************
;                  Binary To BCD Conversion Routine
;      This routine converts a 16 Bit binary Number to a 5 Digit
;      This routine converts a 24 Bit binary Number to an 8 Digit
; BCD Number. This routine is useful since PIC16C55 & PIC16C57
; have  two 8 bit ports and one 4 bit port ( total of 5 BCD digits)
;
;       The 16 bit binary number is input in locations H_byte and
; L_byte with the high byte in H_byte.
;       The 5 digit BCD number is returned in R0, R1 and R2 with R0
; containing the MSD in its right most nibble.
;
;   Performance :			16 bit	24 bit
;               Program Memory  :       35	
;               Clock Cycles    :       885	2100 approx
;
; This file is based on the file BCD.ASM found on www.crownhill.co.uk
; and http://ww1.microchip.com/downloads/en/AppNotes/00526e.pdf
;*******************************************************************;
;
;

B2_BCD  bcf     STATUS,0                ; clear the carry bit
	movlw   .32	; number of bits to convert.
	
;	movlw   .24	; number of bits to convert. ; this did not work, left result in wrong place.
	movwf   BCDcount
	clrf    BCDop0
	clrf    BCDop1
	clrf    BCDop2
	clrf    BCDop3
	clrf    BCDop4

BCDloop16
	bcf	STATUS,C
	btfsc	BCDip3,7
	bsf	STATUS,C
	rlf     BCDip0
	rlf     BCDip1
	rlf     BCDip2
	rlf     BCDip3
	
	rlf     BCDop0
	rlf     BCDop1
	rlf     BCDop2
	rlf     BCDop3
	rlf     BCDop4
;
	decfsz  BCDcount
	goto    BCDadjDEC
	RETLW   0
;
BCDadjDEC  
;	movlw   BCDop4
;	movwf   FSR
;	call    BCDadjBCD

	movlw   BCDop0
	movwf   FSR
	call    BCDadjBCD

	movlw   BCDop1
	movwf   FSR
	call    BCDadjBCD
;
	movlw   BCDop2
	movwf   FSR
	call    BCDadjBCD
;
	movlw   BCDop3
	movwf   FSR
	call    BCDadjBCD

	movlw   BCDop4
	movwf   FSR
	call    BCDadjBCD


;
	goto    BCDloop16
;

BCDadjBCD  
	movlw   0x03
	addwf   0,W
	movwf   BCDtemp
	btfsc   BCDtemp,3       ; test if result > 7
	movwf   0
	movlw   0x30
	addwf   0,W
	movwf   BCDtemp
	btfsc   BCDtemp,7       ; test if result > 7
	movwf   0               ; save as MSD
	RETLW   0


	end


